.has-menu .menu {
  display: block;
  list-style: none;
  opacity: 0;
  margin: 0;
  padding: 0;
  position: absolute;
  transition: visibility 0.1s, opacity 0.1s;
  visibility: hidden;
  z-index: 110;
}
.has-menu:hover > .menu {
  visibility: visible;
  opacity: 1;
}
/*# sourceMappingURL=menus.css.map */