/****************************************************************************
 * Shared rules for all infoboxes
 ****************************************************************************/
.infobox {
  background: #F9F9F9;
  border: 1px #666666 solid;
  border-radius: 3px;
  box-shadow: 0px 0px 4px rgba(0, 0, 0, 0.5);
  display: block;
  font-size: 12px;
  overflow: hidden;
  position: absolute;
  min-width: 40em;
  max-width: 50em;
  min-height: 10em;
  z-index: 120;
}
.infobox a {
  text-decoration: none;
}
.infobox a:hover {
  text-decoration: underline;
}
.infobox h2,
.infobox .infobox-header-text {
  color: inherit;
  font-size: 1.4em;
  font-weight: bold;
  margin: 0 0 0.5em 0;
  padding: 0;
  vertical-align: top;
}
.infobox h2 a,
.infobox .infobox-header-text a {
  color: inherit;
}
.infobox p {
  margin: 0.5em 0;
  padding: 0;
}
.infobox-extra {
  box-sizing: border-box;
  padding: 0.5em;
  width: 100%;
}
.infobox-hover-item,
.infobox-item {
  margin: 0.5em 0;
}
.infobox-hover-item .fa,
.infobox-item .fa {
  min-width: 16px;
  text-align: right;
}
.infobox-hover-item {
  position: relative;
}
.infobox-hover-item.infobox-hover-item-anchor-has-small-icon .infobox-hover-item-content {
  padding-left: 26px;
  padding-right: 26px;
}
.infobox-hover-item.infobox-hover-item-opened .infobox-hover-item-content,
.infobox-hover-item.infobox-hover-item-opened .infobox-hover-item-anchor {
  z-index: 121;
}
.infobox-hover-item.infobox-hover-item-opened .infobox-hover-item-content {
  display: block;
}
.infobox-hover-item-content {
  border: 1px solid #666;
  background-color: white;
  display: none;
  margin: -4px 0 0 -6px;
  padding: 24px 6px 4px 6px;
  position: absolute;
  top: 0;
  left: 0;
}
.infobox-hover-item-anchor {
  cursor: pointer;
  position: relative;
}
.infobox-hover-item-anchor label {
  border-bottom: 1px dotted #666;
}
.infobox-links a {
  color: #0000C0;
  margin-right: 2em;
}
.infobox-pic {
  display: inline-block;
  vertical-align: top;
}
.infobox-pic img {
  border-radius: 2px 0 0 2px;
}
.infobox-scrollable-section {
  padding-left: 20px;
  position: relative;
}
.infobox-scrollable-section:after {
  content: "";
  position: absolute;
  bottom: 0;
  left: 0;
  width: 100%;
  height: 2em;
  background: linear-gradient(to bottom, rgba(249, 249, 249, 0), rgba(249, 249, 249, 0.6) 100%);
}
.infobox-scrollable-section:hover:after {
  display: none;
}
.infobox-scrollable-section:hover .infobox-scrollable-section-content {
  overflow-y: scroll;
}
.infobox-scrollable-section .infobox-scrollable-section-content {
  max-height: 8em;
  overflow: hidden;
}
.infobox-small-detail {
  font-size: 10px;
}
.infobox-text {
  box-sizing: border-box;
  display: inline-block;
  padding: 0.75em;
  position: relative;
  vertical-align: top;
  width: 100%;
  word-wrap: break-word;
}
.infobox-text-section {
  margin-top: 1.5em;
}
.infobox-text-section > pre {
  font-size: 12px;
  white-space: pre-wrap;
}
/****************************************************************************
 * User infobox
 ****************************************************************************/
.user-infobox {
  min-height: 128px;
  /*
   * If there's extra content provided by extensions, the avatar image should
   * have a flat bottom. If not, round the bottom-left as well as the top-left.
   */
}
.user-infobox .infobox-pic {
  height: 128px;
  width: 128px;
}
.user-infobox .has-extra-content .infobox-pic img {
  border-radius: 2px 0 0 0;
}
.user-infobox .infobox-extra {
  border-top: 1px solid #666;
}
.user-infobox .infobox-text {
  min-height: 128px;
  min-width: 20em;
  max-width: 40em;
  width: auto;
}
.user-infobox .infobox-text .infobox-links {
  position: absolute;
  bottom: 0.75em;
}
.user-infobox h2 .username {
  color: #888888;
  font-size: 12px;
  font-weight: normal;
}
/****************************************************************************
 * Bug infobox
 ****************************************************************************/
.bug-infobox .bug-infobox-id {
  float: right;
}
.bug-infobox .bug-infobox-id a {
  color: #888;
  font-size: 90%;
  font-weight: normal;
}
/****************************************************************************
 * Review Request infobox
 ****************************************************************************/
.review-request-infobox .infobox-pic {
  border-radius: 0;
  vertical-align: middle;
}
.review-request-infobox .review-request-infobox-header {
  margin-bottom: 1em;
}
.review-request-infobox .review-request-infobox-id {
  float: right;
}
.review-request-infobox .review-request-infobox-id a {
  color: #888;
  font-size: 90%;
  font-weight: normal;
}
.review-request-infobox .review-request-infobox-label-draft {
  background-color: #AAE29B;
  border: 1px #51b936 solid;
  border-radius: 5px;
  color: #005500;
  font-size: 75%;
  font-weight: normal;
  padding: 0.1em 0.35em 0.25em 0.35em;
}
.review-request-infobox .review-request-infobox-label-submitted {
  background-color: #FFFFD0;
  border: 1px #b6b600 solid;
  border-radius: 5px;
  color: #777700;
  font-size: 75%;
  font-weight: normal;
  padding: 0.1em 0.35em 0.25em 0.35em;
}
.review-request-infobox .review-request-infobox-label-discarded {
  background-color: #DDDDDD;
  border: 1px #9d9d9d solid;
  border-radius: 5px;
  color: #555555;
  font-size: 75%;
  font-weight: normal;
  padding: 0.1em 0.35em 0.25em 0.35em;
}
.review-request-infobox .review-request-infobox-review-summary {
  background: #F0F0F0;
  border-top: 1px #CCC solid;
  border-radius: 0 0 2px 2px;
  margin-left: -0.75em;
  margin-right: -0.75em;
  margin-bottom: -0.75em;
  padding: 0.75em;
}
.review-request-infobox .review-request-infobox-review-summary.has-ship-its {
  background: #bcf592;
  border-color: #8fd051;
}
.review-request-infobox .review-request-infobox-review-summary.has-issues {
  background: #fff4B0;
  border-color: #e3c300;
}
.review-request-infobox .review-request-infobox-review-summary a {
  color: #0000C0;
}
.review-request-infobox .review-request-infobox-review-summary ul {
  list-style: none;
  margin: 0;
  padding: 0;
  vertical-align: bottom;
}
.review-request-infobox .review-request-infobox-review-summary li {
  display: inline-block;
  padding-left: 1.5em;
}
.review-request-infobox .review-request-infobox-review-summary li:first-child {
  padding-left: 0;
}
.review-request-infobox .review-request-infobox-review-summary li .rb-icon {
  vertical-align: text-top;
}
/*# sourceMappingURL=infoboxes.css.map */