{% load djblets_utils i18n gravatars rb_extensions %}

<div id="headerbar">
{% include "base/branding.html" %}
 <div id="nav_toggle"><span class="fa fa-navicon"></span></div>
 <ul id="accountnav">
{% include "base/_nav_support_menu.html" %}
{% header_action_hooks %}
{% header_dropdown_action_hooks %}
 </ul>

 <div id="search">
{% if siteconfig_settings.search_enable %}
  <form method="get" action="{% url 'search' %}">
{% endif %}
   <input type="search" name="q" placeholder="{% if siteconfig_settings.search_enable %}{% trans "Search" %}{% else %}{% trans "Jump to..." %}{% endif %}" id="search_field" {% attr "value" %}{{query}}{% endattr %} />
{% if siteconfig_settings.search_enable %}
  </form>
{% endif %}
 </div>
</div>
