{% extends "admin/change_form.html" %}
{% load pipeline %}

{% block scripts-post %}
{{block.super}}
{%  javascript 'djblets-forms' %}
{%  javascript 'widgets' %}
{% endblock %}

{% block extrastyle %}
{{block.super}}
{%  stylesheet 'djblets-forms' %}
{%  stylesheet 'widgets' %}
{% endblock %}
