/*
 * Any changes to this need to be reflected in _resizeLayout in
 * reviewRequestEditorView.js
 */
/****************************************************************************
 * Review request box
 ****************************************************************************/
.review-request {
  background: none;
  border-radius: 6px;
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
}
.review-request textarea {
  border: 1px #b8b5a0 solid;
  height: 1.5rem;
  overflow: hidden;
}
.review-request .inline-editor-form input[type='text']:not(:focus) {
  box-shadow: 0px 0px 4px rgba(255, 0, 0, 0.75);
}
.review-request .required-flag {
  color: darkred;
  font-size: 10px;
  font-weight: bold;
  padding: 0 2px;
}
.review-request-header {
  border-bottom: 1px #a88d32 solid;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  .review-request-header {
    text-align: center;
  }
}
.review-request-body {
  background-color: #FEFADF;
  background-repeat: repeat-x;
  background-image: linear-gradient(to bottom, #fffdf3, #FEFADF);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fffdf3', endColorstr='#FEFADF', GradientType=0);
  border: 1px #9E9A7F solid;
  border-top: 0;
  border-radius: 0 0 6px 6px;
  clear: both;
  padding: 0.5rem;
}
.review-request-body .field-container {
  position: relative;
}
.banner pre.field,
.review-request-body pre.field {
  background-color: white;
  border: 1px #b8b5a0 solid;
  margin: 10px 0 0 0;
  padding: 10px;
  white-space: pre-wrap;
}
.banner pre.field .codehilite,
.review-request-body pre.field .codehilite {
  border-left: 0;
  border-right: 0;
  border-radius: 0;
  margin: -4px -10px;
}
.review-request-section {
  padding: 10px;
  position: relative;
}
.review-request-section-label {
  color: #575012;
  font-size: 13px;
  font-weight: bold;
}
.review-request-summary p {
  margin: 10px 0 0 0;
}
.review-request-summary #field_summary {
  display: inline;
}
.review-request-summary #field_summary a {
  color: black;
  text-decoration: none;
}
.review-request-summary .review-request-section-label,
.review-request-summary #field_summary {
  font-size: 14px;
}
.review-request-summary .review-request-section-label {
  /* Hide the "Summary:" label on mobile. */
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  .review-request-summary .review-request-section-label {
    display: none;
  }
}
.review-request-summary .rb-icon-edit {
  vertical-align: text-bottom;
}
#review-request-main {
  margin-right: 310px;
  position: relative;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  #review-request-main {
    margin-right: 0;
  }
}
#review-request-extra {
  clear: both;
}
#review-request-warning {
  background-color: #F5A9A9;
  border: 1px solid #FF0000;
  display: none;
  font-weight: bold;
  padding: 5px;
  text-align: center;
}
/****************************************************************************
 * Review request details pane
 ****************************************************************************/
#review-request-details {
  border-spacing: 0px;
  float: right;
  width: 300px;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  #review-request-details {
    /* Give the details pane its own full row above the other sections. */
    float: none;
    width: auto;
  }
}
#review-request-details a {
  color: #0000EE;
  text-decoration: none;
}
#review-request-details tbody td,
#review-request-details tbody th,
#review-request-details tbody .field {
  vertical-align: text-top;
}
#review-request-details tbody td,
#review-request-details tbody th {
  padding: 0 0 15px 10px;
}
#review-request-details tbody th {
  text-align: right;
  white-space: nowrap;
}
#review-request-details tbody .editicon {
  /* Undo style added in common.less */
  margin-left: 0;
}
#review-request-details tbody .field {
  display: inline-block;
  margin-right: 4px;
  word-wrap: break-word;
}
#review-request-details tbody .field:empty {
  margin-right: 0;
}
#review-request-details thead.first-fieldset th {
  padding-top: 0;
}
#review-request-details thead th {
  color: #575012;
  font-size: 13px;
  padding: 20px 0 10px 0;
  text-align: left;
}
#review-request-details th {
  color: #39350B;
  font-weight: bold;
}
#review-request-details .buttons {
  display: block;
  margin-bottom: 10px;
  padding: 6px 0 0 0;
}
#review-request-details .buttons input {
  font-size: 11px;
}
/****************************************************************************
 * Review request actions
 ****************************************************************************/
.review-request-actions-container {
  background: #E5D7A8;
  border-color: #a88d32;
  border-radius: 5px 5px 0 0;
  border-style: solid;
  border-width: 1px 1px 0 1px;
  box-sizing: border-box;
  min-height: 26px;
  overflow: hidden;
}
.review-ui-box .review-request-actions-container {
  border-radius: 5px;
}
.review-ui-box.has-review-ui-box-content .review-request-actions-container {
  border-radius: 5px 5px 0 0;
}
.review-request-actions-left {
  float: left;
}
.review-request-actions-right-container,
.review-request-actions-right {
  float: right;
}
.review-request-actions {
  box-sizing: border-box;
  list-style: none;
  margin: 0;
  padding: 0;
  white-space: nowrap;
}
.review-request-action {
  float: left;
}
.review-request-action:hover {
  background-color: #d7c179;
}
.review-request-action:active {
  background-color: #ceb359;
}
.review-request-action a {
  color: black;
  cursor: pointer;
  display: block;
  margin: 0;
  line-height: 16px;
  text-decoration: none;
  padding: 5px 10px;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  .review-request-action a {
    /* Give some extra room for tapping. */
    padding: 10px 10px;
  }
}
.review-request-action .menu {
  background: #ceb359;
  border: 1px #a88d32 solid;
  border-radius: 0 0 6px 6px;
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  list-style: none;
  margin: 0;
  float: none;
}
.review-request-action .menu li {
  background: #E5D7A8;
  border: 0;
  float: none;
  margin: 0;
  padding: 0;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  .review-request-action .menu li {
    /* Give some extra room for tapping. */
    padding: 10px;
  }
}
.review-request-action .menu li:last-child {
  border-radius: 0 0 6px 6px;
}
.review-request-action .menu li:hover {
  background-color: #d7c179;
}
.review-request-action-icon {
  display: inline-block;
}
.review-request-action-icon > a:first-child {
  line-height: inherit;
  padding: 5px 5px;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  .review-request-action-icon > a:first-child {
    padding: 10px;
  }
}
/*
 * The main text-based actions turn into a drop-down menu on mobile.
 *
 * The following rules do several things:
 * - On desktop, we don't want to have any kind of hover/active highlighting
 *   on the review-request-actions-right-container (which is itself an "action"
 *   due to it's menu-ness).
 * - Hide the mobile actions menu "..." icon on desktop, show on mobile.
 * - Turn .review-request-actions-right into a drop-down menu, but a special
 *   one that lists its items horizontalli (with wrapping), rather than a
 *   vertical layout.
 */
.review-request-actions-right-container > .review-request-action:hover {
  background-color: inherit;
}
a.mobile-actions-menu-label {
  display: none;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  a.mobile-actions-menu-label {
    display: inline-block;
  }
  .review-request-actions-right-container > .review-request-action:hover {
    background-color: #d7c179;
  }
  .review-request-actions-right {
    display: block;
    opacity: 0;
    padding: 0;
    position: absolute;
    transition: visibility 0.1s, opacity 0.1s;
    visibility: hidden;
    z-index: 110;
    background: #ceb359;
    border: 1px #a88d32 solid;
    border-radius: 0 0 6px 6px;
    box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
    list-style: none;
    margin: 0;
    background: #E5D7A8;
    left: 0;
    white-space: normal;
    width: 100%;
  }
  .review-request-actions-right li {
    background: #E5D7A8;
    border: 0;
    float: none;
    margin: 0;
    padding: 0;
  }
  .review-request-actions-right li:last-child {
    border-radius: 0 0 6px 6px;
  }
  .review-request-actions-right li:hover {
    background-color: #d7c179;
  }
  .review-request-actions-right > li {
    display: inline-block;
  }
  .has-menu:hover .review-request-actions-right {
    opacity: 1;
    visibility: visible;
  }
}
@media screen and (max-width: 720px) and screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) and screen and (max-width: 720px), screen and (max-width: 720px) and screen and (max-device-width: 720px) and (orientation: landscape), screen and (max-device-width: 720px) and (orientation: landscape) and screen and (max-device-width: 720px) and (orientation: landscape) {
  .review-request-actions-right li {
    /* Give some extra room for tapping. */
    padding: 10px;
  }
}
/****************************************************************************
 * Review request tabs
 ****************************************************************************/
.review-request-tabs {
  background-color: #E5D7A8;
  border-color: #9E9A7F;
  border-radius: 6px 6px 0 0;
  border-style: solid;
  border-width: 1px 1px 0 1px;
  float: right;
  margin-left: 1.5rem;
  overflow: hidden;
}
.review-ui-box .review-request-tabs .active {
  /*
     * The active tab in review UIs has a slightly different color to match the
     * review UI header color.
     */
  background-color: #FEFADF;
}
.review-request-tabs li {
  border-bottom: 1px #a88d32 solid;
  min-width: 4rem;
  text-align: center;
}
.review-request-tabs li.active {
  background: #fffdf3;
  border-color: #9E9A7F;
  border-bottom-color: transparent;
  border-style: solid;
  border-width: 0 1px 1px 1px;
}
.review-request-tabs li.active:first-child {
  border-left: 0;
}
.review-request-tabs li.active:last-child {
  border-right: 0;
}
/****************************************************************************
 * Banners
 ****************************************************************************/
#discard-banner,
#submitted-banner {
  background: #E0E0E0;
  border: 1px #808080 solid;
}
#draft-banner,
#discard-banner,
#submitted-banner {
  border-top: 0;
  border-left: 0;
  border-right: 0;
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  margin-top: -1.5em;
  margin-left: -1.5em;
  margin-right: -1.5em;
  margin-bottom: 1.5em;
  padding: 1.5em;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  #draft-banner .banner-actions,
  #discard-banner .banner-actions,
  #submitted-banner .banner-actions {
    margin-bottom: 2em;
  }
}
#draft-banner p label,
#discard-banner p label,
#submitted-banner p label {
  display: block;
  line-height: 16px;
  vertical-align: top;
  padding-top: 1em;
}
#draft-banner > p,
#discard-banner > p,
#submitted-banner > p {
  margin: 1em 0 0 0;
}
#draft-banner .interdiff-link {
  margin-top: 1em;
}
/****************************************************************************
 * Updates Bubble
 ****************************************************************************/
#updates-bubble {
  background: #FFF1AB;
  border-top: 1px #888866 solid;
  border-left: 1px #888866 solid;
  bottom: 0;
  border-radius: 10px 0 0 0;
  box-shadow: -1px -1px 2px rgba(0, 0, 0, 0.15);
  font-size: 110%;
  padding: 1em;
  position: fixed;
  right: 0;
  z-index: 130;
}
#updates-bubble a,
#updates-bubble a:visited {
  color: #0000CC;
  text-decoration: none;
}
#updates-bubble #updates-bubble-buttons {
  margin-left: 2em;
}
/*# sourceMappingURL=review-request.css.map */