{% load djblets_utils %}
<html>
 <body style="font-family: Verdana, Arial, Helvetica, Sans-Serif;">
  <table bgcolor="#f9f3c9" width="100%" cellpadding="8" style="border: 1px #c9c399 solid;">
   <tr>
    <td>
     This is an automatically generated e-mail.
    </td>
   </tr>
  </table>

  <p>Hi {{user|user_displayname}},</p>
  <p>
   Your password has been successfully changed on
   <a href="{{server_url}}">{{server_url}}</a>.
   If you did not change your password, please contact a server administrator
   immediately.
  </p>
{% if has_api_tokens %}
  <p>
   You currently have API dokens. Changing your password does reset them. If
   you wish to invalidate your API tokens, you must do that manually at
   <a href="{{api_token_url}}">{{api_token_url}}</a>.
  </p>
{% endif %}
 </body>
</html>
