@import (reference) "../defs.less";


.dnd-overlay {
  background: @dnd-overlay-background;
  border: @dnd-overlay-border;
  box-sizing: border-box;
  color: @dnd-overlay-text-color;
  font-size: @dnd-overlay-font-size;
  position: absolute;
  text-align: center;
  vertical-align: middle;
  z-index: @z-index-dnd-overlay;
  left: 0;
  top: 0;
  width: 100%;
  height: 100%;
}

.dnd-overlay-highlight {
  background: lighten(@dnd-overlay-background, 10%);
}
