/**
 * Base class for the model for a page.
 *
 * This doesn't provide any functionality by itself, but may be used in the
 * future for introducing additional logic for pages.
 *
 * This is intended for use by page views that are set by
 * :js:class:`RB.PageManager`.
 */
RB.Page = Backbone.Model.extend();
