.dnd-overlay {
  background: rgba(0, 0, 0, 0.8);
  border: solid 2px grey;
  box-sizing: border-box;
  color: #dddddd;
  font-size: 150%;
  position: absolute;
  text-align: center;
  vertical-align: middle;
  z-index: 210;
  left: 0;
  top: 0;
  width: 100%;
  height: 100%;
}
.dnd-overlay-highlight {
  background: rgba(26, 26, 26, 0.8);
}
/*# sourceMappingURL=dnd-uploader.css.map */