'use strict';

/**
 * Displays a review UI for Markdown files.
 */
RB.MarkdownReviewableView = RB.TextBasedReviewableView.extend({
  className: 'markdown-review-ui'
});

//# sourceMappingURL=markdownReviewableView.js.map