{% extends "js/tests_base.html" %}
{% load pipeline %}

{% block title %}JavaScript Unit Tests{% endblock %}

{% block scripts-test %}
{%  javascript "js-tests" %}
{%  javascript "djblets-js-tests" %}
{% endblock %}
