.split-btn {
  cursor: pointer;
  display: inline-block;
  position: relative;
  transition: 0.25s border linear;
}
.split-btn.drop-down-shown .split-btn-alternatives,
.split-btn.drop-up-shown .split-btn-alternatives {
  opacity: 1;
}
.split-btn.drop-down-shown {
  border-radius: 4px 4px 0 0;
  border-bottom-color: transparent;
}
.split-btn.drop-down-shown .split-btn-alternatives .btn {
  border-bottom: 0;
}
.split-btn.drop-down-shown .split-btn-alternatives .btn:last-child {
  border-bottom: 1px solid #bbb;
  border-radius: 0 0 4px 4px;
}
.split-btn.drop-up-shown {
  border-radius: 0 0 4px 4px;
  border-top-color: transparent;
}
.split-btn.drop-up-shown .split-btn-alternatives .btn {
  border-top: 0;
}
.split-btn.drop-up-shown .split-btn-alternatives .btn:first-child {
  border-radius: 4px 4px 0 0;
  border-top: 1px solid #ccc;
}
.split-btn-alternatives {
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  cursor: pointer;
  opacity: 0;
  position: absolute;
  text-align: left;
  transition: 0.25s opacity linear;
  white-space: nowrap;
}
.split-btn-alternatives .btn {
  border-radius: 0;
  display: block;
}
a.btn {
  text-decoration: none;
}
.btn,
input[type="button"],
input[type="submit"] {
  cursor: pointer;
  display: inline-block;
  border-bottom-color: #bbb;
  border-radius: 4px;
  border: 1px solid #ccc;
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  color: #333;
  font-weight: normal;
  font-size: 13px;
  line-height: normal;
  padding: 3px 8px;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.75);
  transition: 0.1s linear all;
  /*
   * Apply this after all the above rules in order to turn off the main
   * background. This will let each segment manage its own background,
   * for the purposes of hovering.
   */
}
.btn[disabled],
input[type="button"][disabled],
input[type="submit"][disabled] {
  cursor: default;
  background-image: none;
  filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
  opacity: 0.65;
  box-shadow: none;
}
.btn:active,
input[type="button"]:active,
input[type="submit"]:active {
  box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.25), 0 1px 2px rgba(0, 0, 0, 0.05);
}
.btn,
input[type="button"],
input[type="submit"],
.btn .btn-segment,
input[type="button"] .btn-segment,
input[type="submit"] .btn-segment {
  background-color: #e6e6e6;
  background-repeat: no-repeat;
  background-image: linear-gradient(to bottom, #ffffff, #ffffff 25%, #e6e6e6);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffff', endColorstr='#e6e6e6', GradientType=0);
}
.btn:hover,
input[type="button"]:hover,
input[type="submit"]:hover,
.btn .btn-segment:hover,
input[type="button"] .btn-segment:hover,
input[type="submit"] .btn-segment:hover,
.btn.hover,
input[type="button"].hover,
input[type="submit"].hover,
.btn .btn-segment.hover,
input[type="button"] .btn-segment.hover,
input[type="submit"] .btn-segment.hover {
  background-position: 0 -15px;
  color: #333;
  text-decoration: none;
}
.btn.danger,
input[type="button"].danger,
input[type="submit"].danger,
.btn.danger .btn-segment,
input[type="button"].danger .btn-segment,
input[type="submit"].danger .btn-segment {
  color: white !important;
  background-color: #c43c35;
  background-repeat: repeat-x;
  background-image: linear-gradient(to bottom, #ee5f5b, #c43c35);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ee5f5b', endColorstr='#c43c35', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #c43c35 #c43c35 #882a25;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
}
.btn.disabled,
input[type="button"].disabled,
input[type="submit"].disabled {
  cursor: default;
  background-image: none;
  filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
  opacity: 0.65;
  box-shadow: none;
}
.btn.primary,
input[type="button"].primary,
input[type="submit"].primary,
.btn.default,
input[type="button"].default,
input[type="submit"].default {
  color: white;
  text-shadow: none;
}
.btn.primary,
input[type="button"].primary,
input[type="submit"].primary,
.btn.default,
input[type="button"].default,
input[type="submit"].default,
.btn.primary .btn-segment,
input[type="button"].primary .btn-segment,
input[type="submit"].primary .btn-segment,
.btn.default .btn-segment,
input[type="button"].default .btn-segment,
input[type="submit"].default .btn-segment {
  background-color: #2378db;
  background-repeat: repeat-x;
  background-image: linear-gradient(to bottom, #4A90E2, #2378db);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#4A90E2', endColorstr='#2378db', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #2378db #2378db #195499;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
}
.btn.large,
input[type="button"].large,
input[type="submit"].large {
  font-size: 130%;
  line-height: normal;
  padding: 9px 14px 9px;
  border-radius: 6px;
}
.btn.small,
input[type="button"].small,
input[type="submit"].small {
  padding: 7px 9px 7px;
  font-size: 11px;
}
.btn.btn-segmented,
input[type="button"].btn-segmented,
input[type="submit"].btn-segmented {
  padding: 0;
  background: none;
}
.btn .btn-segment,
input[type="button"] .btn-segment,
input[type="submit"] .btn-segment {
  border-left: 1px solid #ccc;
  display: inline-block;
  padding: 3px 8px;
}
.btn .btn-segment:first-child,
input[type="button"] .btn-segment:first-child,
input[type="submit"] .btn-segment:first-child {
  border-left: 0;
  border-top-left-radius: inherit;
  border-bottom-left-radius: inherit;
}
.btn .btn-segment:last-child,
input[type="button"] .btn-segment:last-child,
input[type="submit"] .btn-segment:last-child {
  border-top-right-radius: inherit;
  border-bottom-right-radius: inherit;
}
/*
 * Force Firefox to render our buttons with our chosen font. Otherwise it will
 * be overridden (on macOS at least) with the system font.
 */
input[type="button"],
input[type="submit"] {
  font-family: Verdana, Arial, Helvetica, Sans-Serif;
}
button::-moz-focus-inner,
input[type=submit]::-moz-focus-inner,
input[type=button]::-moz-focus-inner {
  padding: 0;
  border: 0;
}
/*# sourceMappingURL=buttons.css.map */