.oauth-authorize .box-main {
  padding: 10px;
}
@media screen and (min-width: 720px) {
  .oauth-authorize {
    margin: 0 auto;
    max-width: 100%;
    width: 720px;
  }
}
/*# sourceMappingURL=oauth.css.map */