"""A module exposing the search backends registry."""

from __future__ import unicode_literals

from reviewboard.search.search_backends.registry import SearchBackendRegistry


search_backend_registry = SearchBackendRegistry()
