{% load accounts avatars djblets_utils i18n reviewtags tz %}

<li{% attr "class" %}{% if draft %}draft{% elif reply_is_new %}new-reply{% endif %}{% endattr %}{% if comment_id %} data-comment-id="{{comment_id}}"{% endif %} id="{{anchor_name}}">
 <a class="comment-anchor" name="{{anchor_name}}"></a>
 <div class="floating-anchor"><a href="#{{anchor_name}}" class="fa fa-link fa-flip-horizontal"></a></div>
 <div class="comment-author">
  <label for="{% if draft %}draft{% endif %}comment_{{context_id}}-{{id}}">
   <div class="avatar-container">
{% if use_avatars %}
    {% avatar reply_user 32 %}
{% endif %}
   </div>
   <div class="user-reply-info">
{% if reply_is_new %}
    <div class="rb-icon rb-icon-new-updates" title="{% trans 'New reply' %}"></div>
{% endif %}
    <a href="{% url 'user' reply_user %}" class="user">{% user_profile_display_name reply_user %}</a>
    <a href="#{{anchor_name}}" class="timestamp">
     {% localtime on %}{% blocktrans with timestamp as timestamp and timestamp|date:"c" as timestamp_raw %}<time class="timesince" datetime="{{timestamp_raw}}">{{timestamp}}</time>{% endblocktrans %}{% endlocaltime %}</a>
   </div>
  </label>
 </div>
 <div class="reply-comment">
  <pre id="{% if draft %}draft{% endif %}comment_{{context_id}}-{{id}}"
       data-raw-value="{% normalize_text_for_edit text rich_text %}"
       class="comment-text reviewtext {% rich_text_classname rich_text %}">{{text|render_markdown:rich_text}}</pre>
 </div>
</li>
