@import (reference) "../defs.less";


#integration_list {
  .border-radius(0 0 @box-border-radius @box-border-radius);
}

.integration-configs {
  a {
    color: blue;

    &:hover {
      text-decoration: underline;
    }
  }
}

.integrations-list-page {
  #content-main {
    .border-radius(0 0 @box-border-radius @box-border-radius);
    .box-shadow(@box-shadow);
    float: left;
    width: 100%;
  }
}
