.edit-oauth2-app .help {
  color: #999;
  font-size: 10px !important;
}
.edit-oauth2-app input[type="text"] {
  font-weight: normal;
  margin: 2px 0;
  padding: 2px 3px;
  vertical-align: middle;
}
.edit-oauth2-app ul.errorlist {
  margin: 0 !important;
  padding: 0 !important;
}
.edit-oauth2-app ul.errorlist li {
  background: red url('../../../admin/img/icon_alert.gif') 5px 0.3em no-repeat;
  border: 1px solid red;
  color: white;
  display: block;
  font-size: 12px !important;
  margin: 0 0 3px 0;
  padding: 4px 5px 4px 25px;
}
.edit-oauth2-app ul.errorlist li a {
  color: white;
  text-decoration: underline;
}
td .edit-oauth2-app ul.errorlist {
  margin: 0 !important;
  padding: 0 !important;
}
td .edit-oauth2-app ul.errorlist li {
  margin: 0 !important;
}
.edit-oauth2-app .form-row.errors {
  background: #ffc;
}
.edit-oauth2-app .form-row.errors input,
.edit-oauth2-app .form-row.errors select,
.edit-oauth2-app .form-row.errors textarea {
  border: 1px solid red;
}
.edit-oauth2-app h2 {
  background: url("../../images//title_box_top_bg.png") repeat-x scroll left top #A2BEDC;
  border-bottom: 1px solid #888A85;
  color: black;
  font-size: 11px;
  font-weight: normal;
  margin: 0;
  padding: 0.5em 1em;
  text-align: left;
  text-transform: uppercase;
}
.edit-oauth2-app fieldset {
  background: transparent;
  border: 0;
  border-radius: 0;
  box-shadow: none;
  margin: 0 0 1em 0;
}
.edit-oauth2-app fieldset:first-child > h2 {
  border-top: 0;
}
.edit-oauth2-app fieldset.collapsed {
  margin: 0;
}
.edit-oauth2-app fieldset .description p {
  margin-bottom: 1em;
}
.edit-oauth2-app fieldset .description p:last-child {
  margin-bottom: 0;
}
.edit-oauth2-app fieldset .form-row:last-child {
  border-bottom: 0;
}
.edit-oauth2-app fieldset > h2 {
  border-top: 1px #888A85 solid;
  border-bottom: 1px #888A85 solid;
}
.edit-oauth2-app fieldset.confirmation {
  border-bottom: 1px solid #FFBD09;
  color: black;
  display: block;
  background: #FFFDC9;
}
.edit-oauth2-app fieldset.confirmation a,
.edit-oauth2-app fieldset.confirmation a:visited {
  color: blue;
}
.edit-oauth2-app fieldset.confirmation p {
  margin: 1em 0;
}
.edit-oauth2-app fieldset.confirmation label {
  color: #222222 !important;
}
.edit-oauth2-app fieldset.confirmation li {
  font-size: 12px;
  line-height: 16px;
}
.edit-oauth2-app fieldset.confirmation .cert-info {
  margin: 2em 0;
}
.edit-oauth2-app fieldset.confirmation .cert-info h3 {
  color: black;
  font-weight: bold;
  font-size: 12px;
  margin-bottom: 1em;
}
.edit-oauth2-app fieldset.confirmation .cert-info label {
  display: block;
  float: left;
  font-weight: bold !important;
  margin-left: 2em;
  width: 9em;
}
.edit-oauth2-app fieldset.confirmation .form-row {
  border: 0;
}
.edit-oauth2-app #id_client_secret_container {
  overflow: hidden;
}
#custom_policy_editor {
  margin: 0;
  width: 65em;
}
#custom_policy_editor p {
  font-size: 110%;
  margin: 1em;
}
#custom_policy_editor .CodeMirror {
  border-top: 1px #DDD solid;
  border-bottom: 1px #DDD solid;
}
.config-forms-container .config-forms-page-content .box-main > p,
.config-forms-container .config-forms-page-content .box-main form > p {
  margin-top: 0;
}
.config-forms-container .config-forms-page-content .config-forms-list-item {
  white-space: unset;
}
.config-forms-container .config-forms-page-content .help {
  margin: 1em 2em;
}
#page_authentication .api-tokens .config-forms-list {
  border-bottom: none;
}
#page_authentication .api-tokens .config-forms-list:empty {
  display: none;
}
#page_authentication .api-tokens .config-forms-list:empty + .generate-api-token {
  border-top: 1px solid #AAAAAA;
}
#page_authentication .api-tokens .config-forms-list > :last-child {
  border-bottom: 1px #E9E9E9 solid;
}
#page_authentication .config-site-api-tokens .config-api-token-note {
  color: #888;
}
#page_authentication .config-site-api-tokens .config-api-token-note.empty {
  color: #CCC;
  font-style: italic;
}
#page_authentication .config-site-api-tokens .config-api-token-value {
  font-family: monospace;
  font-size: 120%;
}
#page_authentication .config-site-api-tokens .config-forms-list-action-label-policy-custom {
  color: blue;
  cursor: pointer;
  text-decoration: underline;
}
#page_authentication .config-site-api-tokens .config-forms-list-action-policy {
  text-align: right;
  width: 6.5em;
}
#page_authentication .config-site-api-tokens .rb-icon {
  vertical-align: middle;
}
#page_authentication .generate-api-token {
  background: white;
  border-bottom: 1px solid #AAAAAA;
}
#page_authentication .generate-api-token a {
  color: blue;
  font-size: 90%;
  text-decoration: none;
}
#page_oauth2 .app-entry-wrapper {
  overflow: hidden;
}
#page_oauth2 .config-app-name.disabled a {
  font-style: italic;
}
#page_oauth2 .config-app-name .rb-icon {
  vertical-align: middle;
}
#page_oauth2 .config-app-name a {
  vertical-align: middle;
}
#page_oauth2 .config-forms-list-item {
  white-space: unset;
}
#page_oauth2 .disabled-warning {
  margin-bottom: 1em;
}
#page_oauth2 p {
  line-height: normal;
}
.oauth-add-app {
  margin-top: 1em;
}
.edit-oauth2-app fieldset {
  margin: 0;
  padding: 0;
}
.edit-oauth2-app h2 {
  border-top: 1px solid #888A85;
  border-bottom: 1px solid #888A85;
  margin: 0;
  padding: 4px 10px;
  text-transform: none;
}
.edit-oauth2-app #id_redirect_uris_container li {
  padding: 0.25em 0;
}
.edit-oauth2-app .box-main {
  padding: 0;
}
.edit-oauth2-app .box-main > .errorlist {
  border-bottom: 1px solid #888A85;
  display: block;
}
.edit-oauth2-app .box-main > .errorlist li {
  margin-bottom: 0;
}
.edit-oauth2-app .form-actions {
  padding: 0 10px 10px 10px;
}
.edit-oauth2-app .form-actions a.btn,
.edit-oauth2-app .form-actions input {
  margin-right: 1em;
}
.edit-oauth2-app .form-row {
  margin: inherit -1px;
  padding: auto 10px;
}
.edit-oauth2-app .list-edit-widget ul {
  list-style: none;
}
.edit-oauth2-app .regenerate-secret-button {
  margin-top: 1px;
}
/*# sourceMappingURL=my-account.css.map */