{% load i18n %}

<div id="{{id}}_container">
 {{field}}
{% spaceless %}
 <a href="#" class="copyable-text-input-link" role="button" data-field-id="{{id}}">
 <span class="fa fa-clipboard" title="{% trans 'Copy to clipboard' %}"></span>
 </a>
{% endspaceless %}
 <br>
 <a class="btn danger regenerate-secret-button" href="#">{% trans 'Regenerate client secret' %}</a>
</div>
<script>
(function($) {
    $(document).ready(function() {
        var view = new RB.OAuthClientSecretView({
            el: $('#{{id|escapejs}}_container'),
            apiURL: '{{api_url|escapejs}}'
        });
        view.render();
    });
})($ || window.jQuery || django.jQuery)
</script>
