{% extends "reviews/entries/base.html" %}
{% load i18n %}


{% block entry_classes %}status-updates{% endblock %}
{% block entry_header_classes %}{{entry.state_summary_class}}{% endblock %}


{% block entry_title %}
{%  trans "Checks run" %} ({{entry.state_summary}})
{% endblock entry_title %}


{% block entry_new_label %}{% trans "New status updates" %}{% endblock %}


{% block entry_content %}
<section class="status-update-summary">
{%  for update in entry.status_updates %}
 {{update.summary_html}}
{%  endfor %}
</section>

{%  for update in entry.status_updates %}
{%   include "reviews/entries/_status_update_review_section.html" %}
{%  endfor %}
{% endblock entry_content %}
