#page-container {
  display: none;
}
.jasmine_html-reporter {
  margin: 1.5em;
}
.jasmine_html-reporter .jasmine-banner {
  background: #CDFF9C;
  border-width: 1px;
  border-color: #4d9c00;
  border-style: solid;
  margin-bottom: 10px;
  padding: 8px 10px;
  z-index: 90;
  margin-top: 0;
  padding: 8px;
}
.jasmine_html-reporter .jasmine-banner > h1,
.jasmine_html-reporter .jasmine-banner > p {
  display: inline;
  margin-right: 0.2em;
  padding: 0;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  .jasmine_html-reporter .jasmine-banner p {
    display: block;
    margin: 1em 0;
  }
}
.jasmine_html-reporter .jasmine-banner .banner-actions {
  margin-left: 1em;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  .jasmine_html-reporter .jasmine-banner .banner-actions {
    display: block;
    margin-left: 0;
    margin-top: 1em;
  }
}
.jasmine_html-reporter .jasmine-banner .jasmine-title {
  vertical-align: middle;
  height: 25px;
}
.jasmine_html-reporter .jasmine-banner .jasmine-version {
  color: #666;
  line-height: 25px;
  margin-left: 8px;
  top: 1px;
  vertical-align: middle;
}
.jasmine_html-reporter .jasmine-banner .jasmine-run-options {
  border: 0;
  line-height: 25px;
  margin-right: 0;
}
.jasmine_html-reporter .jasmine-banner .jasmine-run-options .jasmine-trigger {
  border: 0;
  color: #666;
  padding: 8px;
}
.jasmine_html-reporter .jasmine-bar.jasmine-failed {
  background: #deabab;
  color: #222;
}
.jasmine_html-reporter .jasmine-bar.jasmine-passed {
  background: #a6b779;
}
.jasmine_html-reporter .jasmine-bar.jasmine-skipped {
  background: #ddd;
  color: #333;
}
.jasmine_html-reporter .jasmine-bar a {
  color: #444;
}
.jasmine_html-reporter .jasmine-duration {
  color: #444;
}
.jasmine_html-reporter .jasmine-failures .jasmine-spec-detail .jasmine-description {
  background: #b03911;
}
.jasmine_html-reporter .jasmine-spec-detail.jasmine-failed .jasmine-description {
  background: #b7514c;
}
.jasmine_html-reporter .jasmine-spec-detail .jasmine-description {
  padding: 6px 12px;
}
.jasmine_html-reporter .jasmine-summary li.jasmine-passed a {
  color: #5e7d09;
}
.jasmine_html-reporter .jasmine-symbol-summary li.jasmine-passed:before {
  color: #5e7d00;
}
/*# sourceMappingURL=js-tests.css.map */