@import (reference) "../defs.less";

.oauth-authorize .box-main {
  padding: @box-padding;
}

@media screen and (min-width: @mobile-medium-width-720) {
  .oauth-authorize {
    margin: 0 auto;
    max-width: 100%;
    width: @mobile-medium-width-720;
  }
}
