#page-container #page_sidebar.new-review-request-sidebar .repository-selector .search-icon-wrapper {
  margin: 1em;
  position: relative;
  height: 22px;
}
#page-container #page_sidebar.new-review-request-sidebar .repository-selector .search-icon-wrapper .fa-search {
  color: #999;
  font-size: 1.2em;
  position: absolute;
  left: 4px;
  top: 4px;
}
#page-container #page_sidebar.new-review-request-sidebar .repository-selector .search-icon-wrapper .repository-search {
  border: 1px solid #999;
  border-radius: 6px;
  box-shadow: none;
  box-sizing: border-box;
  padding: 2px 4px 4px 20px;
  width: 100%;
  height: 100%;
  z-index: 91;
}
#new-review-request {
  display: none;
  position: relative;
  width: 100%;
}
#new-review-request .main {
  border-radius: 6px;
  height: 55em;
  box-sizing: border-box;
  overflow-y: auto;
  -webkit-overflow-scrolling: touch;
}
#new-review-request .main .hint {
  border-radius: 6px;
  position: relative;
  top: 50%;
  text-align: center;
  vertical-align: middle;
  width: 100%;
}
#new-review-request .new-review-request-container {
  background-color: #FFFFFF;
  border: 1px solid #888A85;
  border-radius: 6px;
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  box-sizing: border-box;
  overflow: hidden;
}
#new-review-request .pre-commit #prompt-for-basedir,
#new-review-request .pre-commit #prompt-for-diff,
#new-review-request .pre-commit #prompt-for-parent-diff,
#new-review-request .pre-commit #prompt-for-change-number,
#new-review-request .pre-commit #processing-diff,
#new-review-request .pre-commit #uploading-diffs,
#new-review-request .pre-commit #error-indicator {
  display: none;
}
#new-review-request .pre-commit #processing-diff .spinner,
#new-review-request .pre-commit #uploading-diffs .spinner {
  width: 16px;
  height: 16px;
  font-size: 16px;
  margin: auto;
}
#new-review-request .pre-commit #error-contents,
#new-review-request .pre-commit #parent-diff-error-contents {
  line-height: 12pt;
  margin-bottom: 1em;
  position: relative;
}
#new-review-request .pre-commit #error-contents img,
#new-review-request .pre-commit #parent-diff-error-contents img {
  margin-right: 5px;
  vertical-align: sub;
}
#new-review-request .pre-commit .input {
  border-radius: 6px;
  background-color: #ffffff;
  position: relative;
  text-align: center;
  vertical-align: middle;
}
#new-review-request .pre-commit .input .startover {
  position: absolute;
  bottom: 0;
  right: 5px;
  line-height: 14pt;
}
#new-review-request .pre-commit .drag-hover {
  border: 1px solid #c6c6c6;
  box-shadow: inset 1px 1px 2px rgba(0, 0, 0, 0.15);
}
#new-review-request .pre-commit .tip {
  margin: 2em 2em 1em 2em;
  text-align: center;
}
#new-review-request .pre-commit .tip a {
  color: #0000C0;
  font-weight: bold;
  text-decoration: none;
}
#new-review-request .pre-commit #prompt-for-parent-diff {
  line-height: 6em;
}
#new-review-request .pre-commit #prompt-for-parent-diff form {
  margin-top: 4em;
}
#new-review-request .pre-commit #error-contents,
#new-review-request .pre-commit #parent-diff-error-contents {
  margin: 0 10em;
}
#new-review-request .pre-commit .input {
  height: 14em;
  line-height: 14em;
  margin: 2em;
  border: 2px dashed #D0D0D0;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  #new-review-request .pre-commit .input {
    height: 6em;
    line-height: 6em;
  }
}
#new-review-request .section-header {
  background-color: #D0E6FF;
  border-bottom: 1px #888A85 solid;
  padding: 0.5em;
  font-size: 120%;
  font-weight: bold;
  text-align: center;
}
#new-review-request .show-repositories {
  color: #444;
  display: block;
  margin-bottom: 1em;
  font-size: 120%;
  text-decoration: none;
  display: none;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  #new-review-request .show-repositories {
    display: block;
  }
}
#new-review-request .branches {
  border-top: 1px #888A85 solid;
  padding: 0;
}
#new-review-request .branches span,
#new-review-request .branches select {
  display: inline-block;
  margin: 0.5em;
  vertical-align: middle;
}
#new-review-request .commit {
  border-bottom: 1px #F0F0F0 solid;
  position: relative;
  padding: 1em;
  cursor: pointer;
}
#new-review-request .commit.disabled {
  color: #888;
  cursor: default;
}
#new-review-request .commit.has-review-request .summary {
  color: #777;
}
#new-review-request .commit .commit-info {
  color: #666;
}
#new-review-request .commit .commit-info .author,
#new-review-request .commit .commit-info .revision {
  margin-right: 1em;
}
#new-review-request .commit .commit-info .author,
#new-review-request .commit .commit-info .revision,
#new-review-request .commit .commit-info .time {
  display: inline-block;
  margin-top: 0.5em;
  white-space: nowrap;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  #new-review-request .commit .commit-info .author,
  #new-review-request .commit .commit-info .revision,
  #new-review-request .commit .commit-info .time {
    display: block;
    float: none;
    margin-top: 0.8em;
  }
}
#new-review-request .commit .commit-info .revision {
  font-family: monospace;
}
#new-review-request .commit .commit-info .time {
  float: right;
}
@media screen and (max-width: 720px), screen and (max-device-width: 720px) and (orientation: landscape) {
  #new-review-request .commit .commit-info .time {
    float: none;
  }
}
#new-review-request .commit .fa {
  color: #888;
  font-size: 1.2em;
}
#new-review-request .commit .fa.fa-code-fork {
  margin-left: 0;
}
#new-review-request .commit .summary {
  font-size: 13px;
  font-weight: 600;
  margin-bottom: 0.6em;
  overflow: hidden;
  vertical-align: bottom;
}
#new-review-request .commit .jump-to-commit {
  color: #71A6E2;
  margin: 0;
  float: right;
  font-size: 1.5em;
}
#new-review-request .commit .progress {
  display: none;
  position: absolute;
  width: 16px;
  right: 1em;
  height: 100%;
  font-size: 16px;
}
#new-review-request .commit:nth-child(even) {
  background-color: #FBFBFB;
}
#new-review-request .commit:nth-child(odd) {
  background-color: #FFFFFF;
}
#new-review-request .commit:hover {
  background-color: #f3f3f3;
}
#new-review-request .post-commit {
  border-bottom: 1px #D0D0D0 solid;
}
#new-review-request .post-commit .error {
  background: #FFE9E9;
  padding: 1em;
  font-size: 110%;
}
#new-review-request .post-commit .error a {
  color: blue;
}
#new-review-request .post-commit .error .error-text {
  color: #CC0000;
}
#new-review-request .files-only {
  margin: 14em;
  text-align: center;
}
#new-review-request .files-only input {
  margin-top: 2em;
}
/*# sourceMappingURL=newReviewRequest.css.map */