{% load avatars i18n %}

<div class="infobox-content {{extra_content|yesno:'has-extra-content,'}}">
 <div class="infobox-details">
{% spaceless %}
{%  if has_avatar %}
  <div class="infobox-pic" class="logo">{% avatar infobox_user 128 %}</div>
{%  endif %}
  <div class="infobox-text">
{% endspaceless %}
{% if show_profile and full_name %}
   <h2><a href="{% url 'user' infobox_user %}">{{full_name}}</a> <span class="username">({{infobox_user.username}})</span></h2>
{% else %}
   <h2><a href="{% url 'user' infobox_user %}">{{infobox_user.username}}</a></h2>
{% endif %}

{% if show_profile %}
{%  if request.user.is_authenticated %}
   <div class="infobox-item email">
    <span class="fa fa-envelope-o"></span>
    <a href="mailto:{{infobox_user.email}}">{{infobox_user.email}}</a>
   </div>
{%  endif %}
   <div class="infobox-hover-item infobox-hover-item-anchor-has-small-icon">
    <div class="infobox-hover-item-content">
     <p class="infobox-small-detail">
{%  blocktrans with infobox_user.last_login|date:"c" as last_login %}
      Last logged in <time class="timesince" datetime="{{last_login}}"></time>
{%  endblocktrans %}
     </p>
     <p class="infobox-small-detail">
{%  blocktrans with infobox_user.date_joined|date:"F jS, Y" as date_joined %}
      Joined {{date_joined}}
{%  endblocktrans %}
     </p>
    </div>
    <span class="infobox-hover-item-anchor">
     <span class="fa fa-clock-o"></span>
     <label class="localtime"><time data-timezone="{{timezone}}"></time></label>
    </span>
   </div>
{% endif %}

   <p class="infobox-small-detail infobox-links">
    <a href="{{review_requests_url}}">{% trans "Review Requests" %}</a>
    <a href="{{reviews_url}}">{% trans "Reviews" %}</a>
   </p>
  </div>
 </div>

{% if extra_content %}
 <div class="infobox-extra">{{extra_content}}</div>
{% endif %}
</div>