'use strict';

/**
 * Base support for displaying a review UI for file attachments.
 */
RB.FileAttachmentReviewableView = RB.AbstractReviewableView.extend({
  commentsListName: 'file_attachment_comments'
});

//# sourceMappingURL=fileAttachmentReviewableView.js.map