#integration_list {
  border-radius: 0 0 6px 6px;
}
.integration-configs a {
  color: blue;
}
.integration-configs a:hover {
  text-decoration: underline;
}
.integrations-list-page #content-main {
  border-radius: 0 0 6px 6px;
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  float: left;
  width: 100%;
}
/*# sourceMappingURL=admin-integrations.css.map */