{% extends "admin/settings.html" %}
{% load djblets_utils i18n staticfiles %}

{% block scripts %}
 <script src="{% static "admin/js/core.js" %}"></script>
 <script src="{% static "admin/js/jquery.js" %}"></script>
 <script src="{% static "admin/js/jquery.init.js" %}"></script>
{%  url "admin:jsi18n" as jsi18nurl %}
 <script src="{{jsi18nurl|default:"../../../jsi18n/"}}"></script>
{{block.super}}
{% endblock %}
